import { Node, Identifier, File, SourceLocation, TSTypeLiteral, ReturnStatement, TaggedTemplateExpression, CallExpression, FunctionDeclaration, FunctionExpression, ArrowFunctionExpression, traverse } from '@babel/types';
import { ParseResult, ParserOptions } from '@babel/parser';
import { BindingTypes, CompilerOptions, RootNode, SourceLocation as SourceLocation$1, TemplateChildNode, ElementNode, ComponentNode } from '@vue/compiler-dom';
import MagicString from 'magic-string';
import { RawSourceMap } from 'source-map-js';
import postcss from 'postcss';

/** This is derived from `@vue/compiler-core` */
declare const VineBindingTypes: {
    /**
     * declared as a prop
     */
    readonly PROPS: BindingTypes.PROPS;
    /**
     * a local alias of a `<script setup>` destructured prop.
     * the original is stored in __propsAliases of the bindingMetadata object.
     */
    readonly PROPS_ALIASED: BindingTypes.PROPS_ALIASED;
    /**
     * a let binding (may or may not be a ref)
     */
    readonly SETUP_LET: BindingTypes.SETUP_LET;
    /**
     * a const binding that can never be a ref.
     * these bindings don't need `unref()` calls when processed in inlined
     * template expressions.
     */
    readonly SETUP_CONST: BindingTypes.SETUP_CONST;
    /**
     * a const binding that does not need `unref()`, but may be mutated.
     */
    readonly SETUP_REACTIVE_CONST: BindingTypes.SETUP_REACTIVE_CONST;
    /**
     * a const binding that may be a ref.
     */
    readonly SETUP_MAYBE_REF: BindingTypes.SETUP_MAYBE_REF;
    /**
     * bindings that are guaranteed to be refs
     */
    readonly SETUP_REF: BindingTypes.SETUP_REF;
    /**
     * a literal constant, e.g. 'foo', 1, true
     */
    readonly LITERAL_CONST: BindingTypes.LITERAL_CONST;
};

type VineBabelRoot = ParseResult<File>;
type LinkedMacroInfo = {
    macroType: 'vineProp';
    macroCall: CallExpression;
    macroMeta: VinePropMeta;
} | {
    macroType: 'vineEmits';
    macroCall: CallExpression;
} | {
    macroType: 'vineSlots';
    macroCall: CallExpression;
};
type VineProcessorLang = 'scss' | 'sass' | 'less' | 'stylus';
type VineStyleLang = 'css' | 'postcss' | VineProcessorLang;
type VineTemplateBindings = Record<string, BindingTypes>;
type BabelFunctionNodeTypes = FunctionDeclaration | FunctionExpression | ArrowFunctionExpression;
declare type HMRCompFnsName = string | null;
interface VineCompilerHooks {
    getCompilerCtx: () => VineCompilerCtx;
    onError: (err: VineDiagnostic) => void;
    onWarn: (warn: VineDiagnostic) => void;
    onBindFileCtx?: (fileId: string, fileCtx: VineFileCtx) => void;
    onValidateEnd?: () => void;
    onAnalysisEnd?: () => void;
    onEnd?: () => void;
}
interface VineCompilerOptions {
    envMode?: string;
    vueCompilerOptions?: CompilerOptions;
    inlineTemplate?: boolean;
    preprocessOptions?: Record<string, any>;
    postcssOptions?: any;
    postcssPlugins?: any[];
}
interface VineStyleMeta {
    lang: VineStyleLang;
    source: string;
    isExternalFilePathSource: boolean;
    range: [number, number] | undefined;
    scoped: boolean;
    fileCtx: VineFileCtx;
    compCtx: VineCompFnCtx;
}
interface VinePropMeta {
    typeAnnotationRaw: string;
    isFromMacroDefine: boolean;
    isBool: boolean;
    isRequired: boolean;
    /** Source code node of given validator function */
    validator?: Node;
    /** Source code node of given default value */
    default?: Node;
    /** Declared identifier AST Node by vineProp */
    declaredIdentifier?: Identifier;
}
interface VineCompilerCtx {
    isRunningHMR: boolean;
    fileCtxMap: Map<string, VineFileCtx>;
    vineCompileErrors: VineDiagnostic[];
    vineCompileWarnings: VineDiagnostic[];
    options: VineCompilerOptions;
}
interface VineUserImport {
    source: string;
    isType: boolean;
    isNamespace?: boolean;
    isDefault?: boolean;
    isUsedInTemplate?: (vineCompFn: VineCompFnCtx) => boolean;
}
interface VineFileCtx {
    readonly fileId: string;
    readonly root: ParseResult<File>;
    readonly originCode: string;
    readonly isCRLF: boolean;
    /**
     * Hot update only executes the
     * markup of the render function
     */
    renderOnly: boolean;
    /**
     * The analysis result of the hot update module,
     * including the function name and scopeId of the component that needs to be updated
     */
    hmrCompFnsName: HMRCompFnsName;
    fileMagicCode: MagicString;
    vineCompFns: VineCompFnCtx[];
    userImports: Record<string, VineUserImport>;
    vueImportAliases: Record<string, string>;
    /** key: `scopeId` => value: `VineStyleMeta` */
    styleDefine: Record<string, VineStyleMeta[]>;
    /**
     * We assume that all import statments are at top of this file,
     * record the end line of these imports
     *
     * To be noticed that the last import statement may take up multiple lines,
     * so we store the its location here.
     */
    importsLastLine?: SourceLocation | null;
    getAstNodeContent: (node: Node) => string;
    getLinkedTSTypeLiteralNodeContent: (node: TSTypeLiteral) => string;
}
interface VineCompFnCtx {
    fnDeclNode: Node;
    fnItselfNode?: BabelFunctionNodeTypes;
    templateSource: string;
    templateReturn?: ReturnStatement;
    templateStringNode?: TaggedTemplateExpression;
    templateAst?: RootNode;
    templateParsedAst?: RootNode;
    isExportDefault: boolean;
    isAsync: boolean;
    /** is web component (customElement) */
    isCustomElement: boolean;
    fnName: string;
    scopeId: string;
    bindings: VineTemplateBindings;
    linkedMacroCalls: LinkedMacroInfo[];
    propsAlias: string;
    props: Record<string, VinePropMeta>;
    propsDefinitionBy: 'annotaion' | 'macro';
    propsFormalParam?: TSTypeLiteral;
    emitsAlias: string;
    emits: string[];
    emitsTypeParam?: TSTypeLiteral;
    /** Store the `defineExpose`'s argument in source code */
    expose?: Node;
    /** Store the `defineOptions`'s argument in source code */
    options?: Node;
    /** Store every slot's props definition */
    slots: Record<string, {
        props: TSTypeLiteral;
    }>;
    slotsNamesInTemplate: string[];
    /** Store `vineModel` defines */
    vineModels: Record<string, {
        varName: string;
        modelModifiersName: string;
        modelOptions: Node | null;
    }>;
    slotsAlias: string;
    hoistSetupStmts: Node[];
    cssBindings: Record<string, string | null>;
    externalStyleFilePaths: string[];
    getPropsTypeRecordStr: (options?: {
        joinStr?: string;
        isNeedLinkedCodeTag?: boolean;
    }) => string;
}
interface VineDiagnostic {
    full: string;
    msg: string;
    location: SourceLocation | null | undefined;
    vineCompFnCtx?: VineCompFnCtx;
    rawVueTemplateLocation?: SourceLocation$1 | null;
}
interface VineCompileCtx {
    compilerHooks: VineCompilerHooks;
    fileCtxCache?: VineFileCtx;
    babelParseOptions?: ParserOptions;
}

declare function findVineCompFnDecls(root: VineBabelRoot): Node[];

declare function compileVineStyle(compilerCtx: VineCompilerCtx, params: {
    vineFileId: string;
    source: string;
    isScoped: boolean;
    scopeId: string;
    preprocessLang?: VineProcessorLang;
    inputSourceMap?: RawSourceMap;
    preprocessOptions?: Record<string, any>;
    postcssOptions?: any;
    postcssPlugins?: any[];
    onErr?: (msg: string) => void;
    onWarn?: (msg: string) => void;
}): Promise<{
    code: string;
    map: RawSourceMap;
    errors: Error[];
    rawResult: postcss.Result<postcss.Document | postcss.Root>;
    dependencies: Set<string>;
} | {
    code: string;
    map: undefined;
    errors: any[];
    rawResult: undefined;
    dependencies: Set<string>;
}>;

type WalkAction = (node: TemplateChildNode, parents: TemplateChildNode[], breakWalk: () => void) => void;
interface WalkActions {
    enter?: WalkAction;
    leave?: WalkAction;
}
interface Walkable {
    type: number;
    children: TemplateChildNode[];
}
declare function walkVueTemplateAst(root: Walkable | undefined, walkActions: WalkActions): void;
declare function isElementNode(node: TemplateChildNode): node is ElementNode;
declare function isComponentNode(node: TemplateChildNode): node is ComponentNode;

declare class ExitTraverseError extends Error {
    constructor();
}
declare const exitTraverse: ExitTraverseError;
declare const _breakableTraverse: typeof traverse;

declare function createCompilerCtx(options?: VineCompilerOptions): VineCompilerCtx;
declare function createVineFileCtx(code: string, fileId: string, vineCompileCtx: VineCompileCtx): VineFileCtx;
declare function doValidateVine(vineCompilerHooks: VineCompilerHooks, vineFileCtx: VineFileCtx, vineCompFnDecls: Node[]): void;
declare function doAnalyzeVine(vineCompilerHooks: VineCompilerHooks, vineFileCtx: VineFileCtx, vineCompFnDecls: Node[]): void;
declare function compileVineTypeScriptFile(code: string, fileId: string, vineCompileCtx: VineCompileCtx, ssr?: boolean): VineFileCtx;

export { type HMRCompFnsName, VineBindingTypes, type VineCompilerCtx, type VineCompilerHooks, type VineCompilerOptions, type VineDiagnostic, type VineFileCtx, type VineCompFnCtx as VineFnCompCtx, type VineProcessorLang, type VinePropMeta, _breakableTraverse, compileVineStyle, compileVineTypeScriptFile, createCompilerCtx, createVineFileCtx, doAnalyzeVine, doValidateVine, exitTraverse, findVineCompFnDecls, isComponentNode, isElementNode, walkVueTemplateAst };
